@@ -1,10 +1,13 @@
 from copy import deepcopy
 
 from django.db import connection as default_django_connection
-from django.db.models import Q, get_model
+from django.db.models import Q, AutoField
 from django.db.models.query import QuerySet
 from django.db.models.constants import LOOKUP_SEP
-from six import string_types
+from django.apps import apps
+get_model = apps.get_model
+import six
+
 
 from querybuilder.fields import FieldFactory, CountField, MaxField, MinField, SumField, AvgField
 from querybuilder.helpers import set_value_for_keypath
@@ -124,6 +127,20 @@ class Join(object):
         if len(self.owner.tables):
             return self.owner.tables[0]
 
+    def get_all_related_objects(self, table):
+        """
+        Fix for django 1.10 to replace deprecated code. Keep support for django 1.7
+        """
+        # Django 1.7 method
+        if hasattr(table.model._meta, 'get_all_related_objects'):
+            return table.model._meta.get_all_related_objects()
+        else:
+            # Django > 1.7
+            return [
+                f for f in table.model._meta.get_fields()
+                if (f.one_to_many or f.one_to_one) and f.auto_created and not f.concrete
+            ]
+
     def set_right_table(self, table):
         """
         Sets the right table for this join clause and try to automatically set the condition
@@ -138,8 +155,11 @@ class Join(object):
             # loop through fields to find the field for this model
 
             # check if this join type is for a related field
-            for field in self.left_table.model._meta.get_all_related_objects():
-                if field.model == self.right_table.model:
+            for field in self.get_all_related_objects(self.left_table):
+                related_model = field.model
+                if hasattr(field, 'related_model'):
+                    related_model = field.related_model
+                if related_model == self.right_table.model:
                     if self.right_table.field_prefix is None:
                         self.right_table.field_prefix = field.get_accessor_name()
                         if len(self.right_table.field_prefix) > 4 and self.right_table.field_prefix[-4:] == '_set':
@@ -149,10 +169,9 @@ class Join(object):
             # check if this join type is for a foreign key
             for field in self.left_table.model._meta.fields:
                 if (
-                    field.get_internal_type() == 'OneToOneField' or
-                    field.get_internal_type() == 'ForeignKey'
+                    field.get_internal_type() == 'OneToOneField' or field.get_internal_type() == 'ForeignKey'
                 ):
-                    if field.rel.to == self.right_table.model:
+                    if field.remote_field.model == self.right_table.model:
                         if self.right_table.field_prefix is None:
                             self.right_table.field_prefix = field.name
                         return
@@ -171,8 +190,11 @@ class Join(object):
             # loop through fields to find the field for this model
 
             # check if this join type is for a related field
-            for field in self.right_table.model._meta.get_all_related_objects():
-                if field.model == self.left_table.model:
+            for field in self.get_all_related_objects(self.right_table):
+                related_model = field.model
+                if hasattr(field, 'related_model'):
+                    related_model = field.related_model
+                if related_model == self.left_table.model:
                     table_join_field = field.field.column
                     # self.table_join_name = field.get_accessor_name()
                     condition = '{0}.{1} = {2}.{3}'.format(
@@ -186,10 +208,9 @@ class Join(object):
             # check if this join type is for a foreign key
             for field in self.right_table.model._meta.fields:
                 if (
-                    field.get_internal_type() == 'OneToOneField' or
-                    field.get_internal_type() == 'ForeignKey'
+                    field.get_internal_type() == 'OneToOneField' or field.get_internal_type() == 'ForeignKey'
                 ):
-                    if field.rel.to == self.left_table.model:
+                    if field.remote_field.model == self.left_table.model:
                         table_join_field = field.column
                         # self.table_join_name = field.name
                         condition = '{0}.{1} = {2}.{3}'.format(
@@ -240,6 +261,7 @@ class Where(object):
         'lt': '<',
         'lte': '<=',
         'contains': 'LIKE',
+        'icontains': 'ILIKE',
         'startswith': 'LIKE',
         'in': 'IN',
     }
@@ -291,7 +313,7 @@ class Where(object):
         :return: the comparison operator from the Where class's comparison_map
         :rtype: str
         """
-        if operator == 'contains':
+        if operator in ('contains', 'icontains'):
             value = '%{0}%'.format(value)
         elif operator == 'startswith':
             value = '{0}%'.format(value)
@@ -324,7 +346,6 @@ class Where(object):
                 operator = '='
 
                 # break apart the field name on double underscores
-                # TODO: do not convert the first double underscore to a .
                 field_parts = field_name.split('__')
                 if len(field_parts) > 1:
                     # get the operator based on the last element split from the double underscores
@@ -332,14 +353,9 @@ class Where(object):
                     operator = self.get_condition_operator(operator_str)
                     if operator is None:
                         operator = '='
-                        field_name = '.'.join(field_parts)
+                        field_name = '__'.join(field_parts)
                     else:
-                        field_name = '.'.join(field_parts[:-1])
-
-                    # if there is more than one double underscore, make the first one a dot
-                    trimmed_field_parts = field_name.split('.')
-                    if len(trimmed_field_parts) > 2:
-                        field_name = '{0}.{1}'.format(trimmed_field_parts[0], '__'.join(trimmed_field_parts[1:]))
+                        field_name = '__'.join(field_parts[:-1])
 
                 # check if we are comparing to null
                 if value is None:
@@ -478,7 +494,7 @@ class Sorter(object):
         # if the specified field is a string with '-' at the beginning
         # the '-' needs to be removed and this sorter needs to be
         # set to desc
-        if isinstance(self.field.field, string_types) and str(self.field.field[0]) == '-':
+        if isinstance(self.field.field, six.string_types) and str(self.field.field[0]) == '-':
             self.desc = True
             self.field.field = self.field.field[1:]
             self.field.name = self.field.name[1:]
@@ -586,6 +602,7 @@ class Query(object):
         self.is_inner = False
         self.with_tables = []
         self._distinct = False
+        self.distinct_ons = []
         self.field_names = []
         self.field_names_pk = None
         self.values = []
@@ -1002,8 +1019,10 @@ class Query(object):
         self._distinct = use_distinct
         return self
 
-    def distinct_on(self):
-        raise NotImplementedError
+    def distinct_on(self, *fields):
+        for field in fields:
+            self.distinct_ons.append(FieldFactory(field))
+        return self
 
     def check_name_collisions(self):
         """
@@ -1134,11 +1153,32 @@ class Query(object):
         row_values = []
         sql_args = []
 
-        for row in rows:
+        # Loop over each row and put the %s placeholder for each field
+        for i, row in enumerate(rows):
+
+            # Build a list of each field's placeholder
             placeholders = []
-            for value in row:
-                sql_args.append(value)
-                placeholders.append('%s')
+
+            # If this is the first row, add casting information so the db knows the field types
+            if i == 0 and hasattr(self.tables[0], 'model'):
+                for field_index, value in enumerate(row):
+                    # Append the value
+                    sql_args.append(value)
+
+                    # Figure out how to cast it
+                    field_object = self.tables[0].model._meta.get_field(field_names[field_index])
+                    db_type = field_object.db_type(self.connection)
+
+                    # Don't cast the pk
+                    if db_type == 'serial':
+                        placeholders.append('%s')
+                    else:
+                        # Cast the placeholder to the data type
+                        placeholders.append('%s::{0}'.format(db_type))
+            else:
+                for value in row:
+                    sql_args.append(value)
+                    placeholders.append('%s')
             row_values.append('({0})'.format(', '.join(placeholders)))
         row_values_sql = ', '.join(row_values)
 
@@ -1160,6 +1200,85 @@ class Query(object):
 
         return self.sql, sql_args
 
+    def get_upsert_sql(
+        self,
+        rows,
+        unique_fields,
+        update_fields,
+        auto_field_name=None,
+        only_insert=False,
+        return_rows=True
+    ):
+        """
+        Generates the postgres specific sql necessary to perform an upsert (ON CONFLICT)
+
+        INSERT INTO table_name (field1, field2)
+        VALUES (1, 'two')
+        ON CONFLICT (unique_field) DO UPDATE SET field2 = EXCLUDED.field2;
+        """
+        ModelClass = self.tables[0].model
+
+        # Use all fields except pk unless the uniqueness constraint is the pk field. Null pk field rows will be
+        # excluded in the upsert method before calling this method
+        all_fields = [field for field in ModelClass._meta.fields if field.column != auto_field_name]
+        if auto_field_name in unique_fields and not only_insert:
+            all_fields = [field for field in ModelClass._meta.fields]
+
+        all_field_names = [field.column for field in all_fields]
+        all_field_names_sql = ', '.join(all_field_names)
+
+        # Convert field names to db column names
+        unique_fields = [
+            ModelClass._meta.get_field(unique_field)
+            for unique_field in unique_fields
+        ]
+        update_fields = [
+            ModelClass._meta.get_field(update_field)
+            for update_field in update_fields
+        ]
+
+        unique_field_names_sql = ', '.join([
+            field.column for field in unique_fields
+        ])
+        update_fields_sql = ', '.join([
+            '{0} = EXCLUDED.{0}'.format(field.column)
+            for field in update_fields
+        ])
+
+        row_values = []
+        sql_args = []
+
+        for row in rows:
+            placeholders = []
+            for field in all_fields:
+                # Convert field value to db value
+                # Use attname here to support fields with custom db_column names
+                sql_args.append(field.get_db_prep_save(getattr(row, field.attname), self.connection))
+                placeholders.append('%s')
+            row_values.append('({0})'.format(', '.join(placeholders)))
+        row_values_sql = ', '.join(row_values)
+
+        if update_fields:
+            self.sql = 'INSERT INTO {0} ({1}) VALUES {2} ON CONFLICT ({3}) DO UPDATE SET {4} {5}'.format(
+                self.tables[0].get_identifier(),
+                all_field_names_sql,
+                row_values_sql,
+                unique_field_names_sql,
+                update_fields_sql,
+                'RETURNING *' if return_rows else ''
+            )
+        else:
+            self.sql = 'INSERT INTO {0} ({1}) VALUES {2} ON CONFLICT ({3}) {4} {5}'.format(
+                self.tables[0].get_identifier(),
+                all_field_names_sql,
+                row_values_sql,
+                unique_field_names_sql,
+                'DO UPDATE SET {0}=EXCLUDED.{0}'.format(unique_fields[0].column),
+                'RETURNING *' if return_rows else ''
+            )
+
+        return self.sql, sql_args
+
     def format_sql(self):
         """
         Builds the sql in a format that is easy for humans to read and debug
@@ -1280,8 +1399,12 @@ class Query(object):
         return sql
 
     def get_distinct_sql(self):
+        if self._distinct and self.distinct_ons:
+            raise ValueError('Cannot combine distinct and distinct_on')
         if self._distinct:
             return 'DISTINCT '
+        if self.distinct_ons:
+            return 'DISTINCT ON ({0}) '.format(', '.join(f.get_sql() for f in self.distinct_ons))
         return ''
 
     def build_from_table(self):
@@ -1420,6 +1543,17 @@ class Query(object):
 
         return self
 
+    def copy(self):
+        """
+        Deeply copies everything in the query object except the connection object is shared
+        """
+        connection = self.connection
+        del self.connection
+        copied_query = deepcopy(self)
+        copied_query.connection = connection
+        self.connection = connection
+        return copied_query
+
     def get_args(self):
         """
         Gets the args for the query which will be escaped when being executed by the
@@ -1579,7 +1713,6 @@ class Query(object):
     def update(self, rows):
         """
         Updates records in the db
-        # TODO: implement this
         """
         if len(rows) == 0:
             return
@@ -1592,6 +1725,94 @@ class Query(object):
         # execute the query
         cursor.execute(sql, sql_args)
 
+    def get_auto_field_name(self, model_class):
+        """
+        If one of the unique_fields is the model's AutoField, return the field name, otherwise return None
+        """
+        # Get auto field name (a model can only have one AutoField)
+        for field in model_class._meta.fields:
+            if isinstance(field, AutoField):
+                return field.column
+
+        return None
+
+    def upsert(self, rows, unique_fields, update_fields, return_rows=False, return_models=False):
+        """
+        Performs an upsert with the set of models defined in rows. If the unique field which is meant
+        to cause a conflict is an auto increment field, then the field should be excluded when its value is null.
+        In this case, an upsert will be performed followed by a bulk_create
+        """
+        if len(rows) == 0:
+            return
+
+        ModelClass = self.tables[0].model
+
+        rows_with_null_auto_field_value = []
+
+        # Get auto field name (a model can only have one AutoField)
+        auto_field_name = self.get_auto_field_name(ModelClass)
+
+        # Check if unique fields list contains an auto field
+        if auto_field_name in unique_fields:
+            # Separate the rows that need to be inserted vs the rows that need to be upserted
+            rows_with_null_auto_field_value = [row for row in rows if getattr(row, auto_field_name) is None]
+            rows = [row for row in rows if getattr(row, auto_field_name) is not None]
+
+        return_value = []
+
+        if rows:
+            sql, sql_args = self.get_upsert_sql(
+                rows,
+                unique_fields,
+                update_fields,
+                auto_field_name=auto_field_name,
+                return_rows=return_rows or return_models
+            )
+
+            # get the cursor to execute the query
+            cursor = self.get_cursor()
+
+            # execute the upsert query
+            cursor.execute(sql, sql_args)
+
+            if return_rows or return_models:
+                return_value.extend(self._fetch_all_as_dict(cursor))
+
+        if rows_with_null_auto_field_value:
+            sql, sql_args = self.get_upsert_sql(
+                rows_with_null_auto_field_value,
+                unique_fields,
+                update_fields,
+                auto_field_name=auto_field_name,
+                only_insert=True,
+                return_rows=return_rows or return_models
+            )
+
+            # get the cursor to execute the query
+            cursor = self.get_cursor()
+
+            # execute the upsert query
+            cursor.execute(sql, sql_args)
+
+            if return_rows or return_models:
+                return_value.extend(self._fetch_all_as_dict(cursor))
+
+        if return_models:
+            ModelClass = self.tables[0].model
+            model_objects = [
+                ModelClass(**row_dict)
+                for row_dict in return_value
+            ]
+
+            # Set the state to indicate the object has been loaded from db
+            for model_object in model_objects:
+                model_object._state.adding = False
+                model_object._state.db = 'default'
+
+            return_value = model_objects
+
+        return return_value
+
     def sql_delete(self):
         """
         Deletes records from the db
@@ -1599,6 +1820,21 @@ class Query(object):
         """
         pass
 
+    def get_count_query(self):
+        """
+        Copies the query object and alters the field list and order by to do a more efficient count
+        """
+        query_copy = self.copy()
+        if not query_copy.tables:
+            raise Exception('No tables specified to do a count')
+
+        for table in query_copy.tables:
+            del table.fields[:]
+
+        query_copy.tables[0].add_field(CountField('*'))
+        del query_copy.sorters[:]
+        return query_copy
+
     def count(self, field='*'):
         """
         Returns a COUNT of the query by wrapping the query and performing a COUNT
@@ -1610,10 +1846,7 @@ class Query(object):
         :return: The number of rows that the query will return
         :rtype: int
         """
-        q = Query(self.connection).from_table(self, fields=[
-            CountField(field)
-        ])
-        rows = q.select(bypass_safe_limit=True)
+        rows = self.get_count_query().select(bypass_safe_limit=True)
         return list(rows[0].values())[0]
 
     def max(self, field):
@@ -1763,20 +1996,24 @@ class QueryBuilderQuerySet(QuerySet):
     class Meta:
         model = None
 
-    def __init__(self, model=None, query=None, using=None):
-        if self.Meta is not None and model is None and hasattr(self.Meta, "model"):
+    def __init__(self, model=None, query=None, using=None, **kwargs):
+        if self.Meta is not None and model is None and hasattr(self.Meta, 'model'):
             model = self.Meta.model
             if isinstance(model, str):
                 model = get_model(*model.split('.', 1))
-        super(QueryBuilderQuerySet, self).__init__(model, query, using)
-        self._queryset = self.model.objects.get_query_set()
+        super(QueryBuilderQuerySet, self).__init__(model, query, using, **kwargs)
+        self._queryset = self.model.objects.get_queryset()
 
     def __getitem__(self, k):
-        return self.get_model_queryset(
-            self._queryset,
-            k.start,
-            k.stop
-        ).all()[k.start:k.stop]
+        if isinstance(k, int):
+            return self.get_model_queryset(self._queryset, k, 1)[0]
+        elif hasattr(k, 'start'):
+            return self.get_model_queryset(
+                self._queryset,
+                k.start,
+                k.stop
+            )
+        return None
 
     def get_model_queryset(self, queryset, offset, limit):
         raise NotImplementedError
@@ -1788,11 +2025,11 @@ class QueryBuilderQuerySet(QuerySet):
 
     def call_field_filter_method(self, field, value, type='filter'):
         field_name = self.get_field_name_from_filter(field)
-        filter_method_name = "{0}__{1}".format(
+        filter_method_name = '{0}__{1}'.format(
             type,
             field_name
         )
-        default_filter_method_name = "{0}__".format(
+        default_filter_method_name = '{0}__'.format(
             type
         )
         filter_method = getattr(self, default_filter_method_name)
@@ -1804,7 +2041,7 @@ class QueryBuilderQuerySet(QuerySet):
         pass
 
     def filter(self, *args, **kwargs):
-        for field, value in kwargs.iteritems():
+        for field, value in six.iteritems(kwargs):
             self.call_field_filter_method(field, value, type='filter')
         return self
 
@@ -1812,7 +2049,7 @@ class QueryBuilderQuerySet(QuerySet):
         pass
 
     def exclude(self, *args, **kwargs):
-        for field, value in kwargs.iteritems():
+        for field, value in six.iteritems(kwargs):
             self.call_field_filter_method(field, value, type='exclude')
         return self
 
@@ -1824,15 +2061,17 @@ class QueryBuilderQuerySet(QuerySet):
 
     def order_by(self, *field_names):
         for field in field_names:
+            if field == 'pk':
+                field = self.model._meta.pk.name
             desc = False
             if field[0] == '-':
                 field = field[1:]
                 desc = True
-            method_name = "{0}__{1}".format(
+            method_name = '{0}__{1}'.format(
                 'order',
                 field
             )
-            default_method_name = "{0}__".format(
+            default_method_name = '{0}__'.format(
                 'order'
             )
             method = getattr(self, default_method_name)
@@ -1843,3 +2082,53 @@ class QueryBuilderQuerySet(QuerySet):
 
     def distinct(self, *field_names):
         raise NotImplementedError
+
+
+class JsonQueryset(QueryBuilderQuerySet):
+
+    def __init__(self, *args, **kwargs):
+        super(JsonQueryset, self).__init__(*args, **kwargs)
+        self.json_query = Query().from_table(self.model)
+
+    def get_model_queryset(self, queryset, offset, limit):
+        return [self.model(**fields) for fields in self.json_query.limit(limit, offset).select()]
+
+    def count(self):
+        return self.json_query.count()
+
+    def order_by(self, *field_names):
+        for field_name in field_names:
+            if field_name == 'pk':
+                field_name = self.model._meta.pk.name
+            reverse = '-' if field_name[0] == '-' else ''
+            field_name = field_name.lstrip('-')
+            parts = field_name.split('->')
+            if len(parts) == 2:
+                self.json_query.order_by('{0}{1}->>\'{2}\''.format(reverse, parts[0], parts[1]))
+            else:
+                self.json_query.order_by('{0}{1}'.format(reverse, field_name))
+        return self
+
+    def filter(self, *args, **kwargs):
+        for key, value in kwargs.items():
+            key = key.replace('__exact', '')
+            parts = key.split('->')
+            if len(parts) == 2:
+                field_key_parts = parts[1].split('__')
+
+                if len(field_key_parts) == 1:
+                    key = '{0}->>\'{1}\''.format(parts[0], field_key_parts[0])
+                else:
+                    key = '{0}->>\'{1}\''.format(parts[0], '__'.join(field_key_parts[0:-1]))
+                    key = '__'.join([key, field_key_parts[-1]])
+                value = six.u('{0}'.format(value))
+            if hasattr(value, 'id'):
+                key = '{0}_id'.format(key)
+                self.json_query.where(**{'{0}'.format(key): value.id})
+            else:
+                self.json_query.where(**{key: value})
+        return self
+
+    def limit(self, *args, **kwargs):
+        self.json_query.limit(*args, **kwargs)
+        return self
